@@ -14,9 +14,10 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 import argparse
+from configparser import ConfigParser
 import logging
+import logging.handlers
 import os
-import six
 import socket
 import sys
 import time
@@ -27,17 +28,8 @@ from keystoneauth1 import session
 
 from novaclient import client
 
-# In python3 SafeConfigParser was renamed to ConfigParser and the default
-# for duplicate options default to true. In case of nova it is valid to
-# have duplicate option lines, e.g. passthrough_whitelist which leads to
-# issues reading the nova.conf
-# https://bugs.launchpad.net/tripleo/+bug/1827775
-if six.PY3:
-    from six.moves.configparser import ConfigParser
-    config = ConfigParser(strict=False)
-else:
-    from six.moves.configparser import SafeConfigParser
-    config = SafeConfigParser()
+
+config = ConfigParser(strict=False)
 
 debug = os.getenv('__OS_DEBUG', 'false')
 
@@ -46,8 +38,17 @@ if debug.lower() == 'true':
 else:
     loglevel = logging.INFO
 
-logging.basicConfig(stream=sys.stdout, level=loglevel)
 LOG = logging.getLogger('nova_wait_for_compute_service')
+LOG_FORMAT = ('%(asctime)s.%(msecs)03d %(levelname)s '
+              '%(name)s %(message)s')
+DATE_FORMAT = '%Y-%m-%d %H:%M:%S'
+LOG.handlers.clear()
+LOG.setLevel(loglevel)
+LOG.propagate = True
+formatter = logging.Formatter(fmt=LOG_FORMAT, datefmt=DATE_FORMAT)
+stream_handler = logging.StreamHandler(sys.stdout)
+stream_handler.setFormatter(formatter)
+LOG.addHandler(stream_handler)
 
 iterations = 60
 timeout = 10
