@@ -17,7 +17,6 @@ import datetime
 import os
 import re
 import shutil
-import six
 import sys
 import yaml
 
@@ -140,9 +139,9 @@ class TemplateLoader(yaml.SafeLoader):
         return collections.OrderedDict(self.construct_pairs(node))
 
 
-TemplateDumper.add_representer(six.text_type,
+TemplateDumper.add_representer(str,
                                TemplateDumper.description_presenter)
-TemplateDumper.add_representer(six.binary_type,
+TemplateDumper.add_representer(bytes,
                                TemplateDumper.description_presenter)
 
 TemplateDumper.add_representer(collections.OrderedDict,
