@@ -15,7 +15,6 @@ import argparse
 import glob
 import os
 import re
-import six
 import sys
 import traceback
 import yaml
@@ -24,7 +23,7 @@ from copy import copy
 
 
 def is_string(value):
-    return isinstance(value, six.string_types)
+    return isinstance(value, str)
 
 
 # Only permit the template alias versions.
@@ -366,23 +365,6 @@ def validate_controller_dashboard(filename, tpl):
     return 0
 
 
-def validate_controller_storage_nfs(filename, tpl, exclude_service=()):
-    control_role_filename = os.path.join(os.path.dirname(filename),
-                                         './Controller.yaml')
-    with open(control_role_filename, 'r') as f:
-        control_role_tpl = yaml.safe_load(f.read())
-
-    control_role_services = control_role_tpl[0]['ServicesDefault']
-    for role in tpl:
-        if role['name'] == 'ControllerStorageNfs':
-            services = [x for x in role['ServicesDefault'] if (x not in exclude_service)]
-            if sorted(services) != sorted(control_role_services):
-                print('ERROR: ServicesDefault in %s is different from '
-                      'ServicesDefault in roles/Controller.yaml' % filename)
-                return 1
-    return 0
-
-
 def validate_hci_role(hci_role_filename, hci_role_tpl):
     role_files = ['HciCephAll', 'HciCephFile', 'HciCephMon', 'HciCephObject']
     if hci_role_filename in ['./roles/' + x + '.yaml' for x in role_files]:
@@ -396,6 +378,7 @@ def validate_hci_role(hci_role_filename, hci_role_tpl):
             if role['name'] == 'HciCephAll':
                 hci_role_services = role['ServicesDefault']
                 hci_role_services.remove('OS::TripleO::Services::CephGrafana')
+                hci_role_services.remove('OS::TripleO::Services::CephIngress')
                 hci_role_services.remove('OS::TripleO::Services::CephMds')
                 hci_role_services.remove('OS::TripleO::Services::CephMgr')
                 hci_role_services.remove('OS::TripleO::Services::CephMon')
@@ -435,6 +418,7 @@ def validate_ceph_role(ceph_role_filename, ceph_role_tpl):
             if role['name'] == 'CephAll':
                 ceph_role_services = role['ServicesDefault']
                 ceph_role_services.remove('OS::TripleO::Services::CephGrafana')
+                ceph_role_services.remove('OS::TripleO::Services::CephIngress')
                 ceph_role_services.remove('OS::TripleO::Services::CephMds')
                 ceph_role_services.remove('OS::TripleO::Services::CephMgr')
                 ceph_role_services.remove('OS::TripleO::Services::CephMon')
@@ -468,7 +452,9 @@ def validate_controller_no_ceph_role(filename, tpl):
             services.append('OS::TripleO::Services::CephMds')
             services.append('OS::TripleO::Services::CephMgr')
             services.append('OS::TripleO::Services::CephGrafana')
+            services.append('OS::TripleO::Services::CephIngress')
             services.append('OS::TripleO::Services::CephMon')
+            services.append('OS::TripleO::Services::CephNfs')
             services.append('OS::TripleO::Services::CephRbdMirror')
             services.append('OS::TripleO::Services::CephRgw')
             if sorted(services) != sorted(control_role_services):
@@ -651,6 +637,17 @@ def validate_docker_service_mysql_usage(filename, tpl):
     return 0
 
 
+def validate_common_service(filename, tpl):
+    # NOTE(bogdando): it doesn't always detect volumes in Heat funcs, like:
+    #   <func>: [{<func>: [FooCommon, volumes]}, [<listed_volumes>]]
+    if 'outputs' not in tpl:
+        print('ERROR: missing outputs for filename: %s' % filename)
+        return 1
+    if validate_ct_volumes(tpl['outputs']):
+        return 1
+    return 0
+
+
 def validate_docker_service(filename, tpl):
     if 'outputs' in tpl and 'role_data' in tpl['outputs']:
         if 'value' not in tpl['outputs']['role_data']:
@@ -658,6 +655,10 @@ def validate_docker_service(filename, tpl):
                   % filename)
             return 1
         role_data = tpl['outputs']['role_data']['value']
+
+        if validate_ct_volumes(role_data):
+            return 1
+
         if list(role_data.keys()) == ['map_merge']:
             merged_role_data = {}
             for part in role_data['map_merge']:
@@ -770,6 +771,55 @@ def validate_docker_service(filename, tpl):
     return 0
 
 
+def validate_ct_volumes(data):
+    '''Ensure we don't have any trailing "/" in data for volumes'''
+    def check_volumes(volumes):
+        if not volumes:
+            return 0
+        result = 0
+        if isinstance(volumes, list):
+            for vol in volumes:
+                if isinstance(vol, dict):
+                    # Avoid 'if', 'get_*' etc
+                    continue
+                elif isinstance(vol, list):
+                    for item in vol:
+                        result += check_volumes(item)
+                elif isinstance(vol, str):
+                    vol_def = vol.split(':')
+                    try:
+                        if vol_def[0][-1] == '/' or vol_def[1][-1] == '/':
+                            print('ERROR: trailing "/" detected'
+                                  ' for {}'.format(vol))
+                            return 1
+                    except IndexError:
+                        # Not a volume definition, ignore it
+                        continue
+        elif isinstance(volumes, dict):
+            # Step into 'list_concat', 'map_*' etc.
+            for item in volumes.values():
+                result += check_volumes(item)
+        return result
+
+    if not data:
+        return 0
+    result = 0
+    for _, item in enumerate(data):
+        if isinstance(data[item], dict):
+            if 'volumes' not in data[item]:
+                result += validate_ct_volumes(data[item])
+            else:
+                result += check_volumes(data[item]['volumes'])
+        elif isinstance(item, list):
+            if 'volumes' not in item:
+                result += validate_ct_volumes(item)
+            else:
+                result += check_volumes(item.index('volumes'))
+        else:
+            continue
+    return result
+
+
 def validate_docker_logging_template(filename, tpl):
     if 'outputs' not in tpl:
         print('ERROR: outputs are missing from: %s' % filename)
@@ -781,6 +831,8 @@ def validate_docker_logging_template(filename, tpl):
         print('ERROR: The file %s is missing the following output(s):'
               ' %s' % (filename, ', '.join(missing_entries)))
         return 1
+    if validate_ct_volumes(tpl['outputs']):
+        return 1
     return 0
 
 
@@ -1122,9 +1174,11 @@ def validate(filename, param_map):
         if re.search(r'^\.\/deployment\/logging\/(files|stdout)\/', filename):
             retval |= validate_docker_logging_template(filename, tpl)
         elif VALIDATE_DOCKER_OVERRIDE.get(filename, False) or (
-                re.search(r'^\.\/deployment\/.+-container-puppet.yaml$', filename) and
+                re.search(r'^\.\/deployment\/.+-container(-puppet)*.yaml$', filename) and
                 VALIDATE_DOCKER_OVERRIDE.get(filename, True)):
             retval |= validate_docker_service(filename, tpl)
+        elif re.search(r'^\.\/deployment\/.+-common.*.yaml$', filename):
+            retval |= validate_common_service(filename, tpl)
 
         if filename.endswith('hyperconverged-ceph.yaml'):
             retval |= validate_hci_compute_services_default(filename, tpl)
@@ -1137,11 +1191,6 @@ def validate(filename, param_map):
                 filename.startswith('./roles/ComputeHCISriov.yaml'):
             retval |= validate_hci_computehci_role(filename, tpl)
 
-        if filename.startswith('./roles/ControllerStorageNfs.yaml'):
-            exclude = [
-                'OS::TripleO::Services::CephNfs']
-            retval |= validate_controller_storage_nfs(filename, tpl, exclude)
-
         if filename.startswith('./roles/ControllerStorageDashboard.yaml'):
             retval |= validate_controller_dashboard(filename, tpl)
         if filename in ['./roles/ComputeOvsDpdk.yaml',
